﻿using System;
using System.ComponentModel;
using System.Linq.Expressions;

namespace IGCheckboxTree
{
    #region FirePropertyChanged Members

    /// <summary>
    /// Class provides custom INotifyPropertyChanged
    /// </summary>
    public class NotificationObject : INotifyPropertyChanged
    {
        /// <summary>
        /// Refreshes View via notification that property has changed
        /// </summary>
        /// <typeparam name="T">Type of object</typeparam>
        /// <param name="action">The property/action</param>
        public void FirePropertyChanged<T>(Expression<Func<T>> action)
        {
            var propertyName = GetPropertyName(action);
            FirePropertyChanged(propertyName);
        }

        /// <summary>
        /// Gets the name of the property to update
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="action"></param>
        /// <returns></returns>
        private static string GetPropertyName<T>(Expression<Func<T>> action)
        {
            var expression = (MemberExpression)action.Body;
            var propertyName = expression.Member.Name;
            return propertyName;
        }

        /// <summary>
        /// Fires the property changed event for the specified property
        /// </summary>
        /// <param name="propertyName">Name of property</param>
        private void FirePropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// Method available to allow one property to update all properties
        /// </summary>
        /// <param name="propertyName">Name of property, use case empty string "" or null</param>
        public void FireAllPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        /// <summary>
        /// PropertyChanged event
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;
    }

    #endregion
}
